require "coup"
require "level"


function love.load()
  G = {}
  G.loadImage = coup.cache(love.graphics.newImage)
  G.animRepo  = coup.sprite.AnimationRepository()

  coup.spriteloader.loadAll("data/sprites", G.animRepo, G.loadImage)

  G.level = Level("level01")

  love.keyboard.setKeyRepeat(true)

  G.titleFont = love.graphics.newFont("data/Norse-Bold.otf", 32)
  G.smallFont = love.graphics.newFont("data/Norse.otf", 30)

  G.sounds = {}
  local sounds = {
    "exit",
    "skit",
    "push",
    "thud",
    "elevator",
    "pick",
    "climp",
    "drop",
    "jump",
    "deep",
    "exit_open"
  }

  G.music = love.audio.newSource("data/music.ogg", "stream")
  G.music:setVolume(0.25)
  G.music:setLooping(true)
  G.music:play()

  for i, s in pairs(sounds) do
    G.sounds[s] = love.audio.newSource("data/audio/" .. s .. ".ogg", "static")
  end
  G.sounds.jump:setVolume(2)

  G.elevator = {}
  G.drop = {}
  G.climb = {}
  for i = 1, 3 do
    G.elevator[i] = G.sounds.elevator:clone()
    G.drop[i] = G.sounds.drop:clone()
    G.climb[i] = G.sounds.climp:clone()
  end
  G.last_elev = 1
  G.last_drop = 1
  G.last_climb = 1

end

function play_climb()
  G.last_climb = G.last_climb + 1
  if G.last_climb == 4 then G.last_climb = 1 end
  G.climb[G.last_climb]:stop()
  G.climb[G.last_climb]:play()
end

function play_elevator()
  G.last_elev = G.last_elev + 1
  if G.last_elev == 4 then G.last_elev = 1 end
  G.elevator[G.last_elev]:stop()
  G.elevator[G.last_elev]:play()
end

function play_drop()
  G.last_drop = G.last_drop + 1
  if G.last_drop == 4 then G.last_drop = 1 end
  G.drop[G.last_drop]:stop()
  G.drop[G.last_drop]:play()

--[[
  G.level.timer = 1
  G.level.tween:tween(G.level, 1.8, {timer = 0}):on_complete(function()
    local dw = G.level.world:getEntityByName("dwarf")
    local tm = dw:getComponent("TilePosition")
    local tx = tm.pos.x * 64
    local ty = tm.pos.y * 64
    for i = 1, 20 do
      local c = math.random(64, 192)
      G.level.particles:spawn(
        tx+32+math.random(0,16), ty+64, --p
        math.random(200, 300), math.random(-10, 10), -- v
        0, math.random(-10,-20), -- a
        0,  -- s
        7.5, -- ds
        math.random(1, 10) / 7, -- life
        {10, 0}, -- damping
        c,c,c)

      G.level.particles:spawn(
        tx+32-math.random(0,16), ty+64, --p
        math.random(-300, -200), math.random(-10, 10), -- v
        0, math.random(-10,-20), -- a
        0,  -- s
        7.5, -- ds
        math.random(1, 10) / 7, -- life
        {10, 0}, -- damping
        c,c,c)
    end
  end)]]
  G.level:dust(1.9)
end


function love.update(dt)
  G.level:update(dt)
end


function love.draw()
  G.level:render()
end


local keyToEvent = {
  x = "jump",
  left = "left",
  right = "right",
  f1 = "toggle_fast",
  down = "pickup",
  up = "up",
  ["return"] = "confirm",
  r = "restart",
  f12 = "skip_level"
}


function love.keypressed(key)
  local event = keyToEvent[key]
  if event == "toggle_fast" then
    G.fast_anim = not G.fast_anim
  elseif event then
    G.level:inputEvent(event)
  end
end
