local Particles = coup.class({})

function Particles:new()
  self.particles = {}
  self.image = G.loadImage("data/smoke.png")
  self.batch = love.graphics.newSpriteBatch(self.image, 512)
  for i = 1, 512 do
    self.particles[i] = {
      x = 0,
      y = 0,
      vx = 0,
      vy = 0,
      ax = 0,
      ay = 0,
      s = 0,
      ds = 0,
      life = 0,
      fulllife = 0,
      damping = {x = 0, y = 0},
      r = 0,
      g = 0,
      b = 0,
      a = 0
    }
  end
  self.count = 0
end


function Particles:spawn(x, y, vx, vy, ax, ay, s, ds, life, damping, r, g, b, a)
  if self.count >= #self.particles then
    return
  end

  damping = damping or 0

  local damping_x 
  local damping_y

  if type(damping) == "table" then
    damping_x = damping[1]
    damping_y = damping[2]
  else
    damping_x = damping
    damping_y = damping
  end

  self.count = self.count + 1
  local p = self.particles[self.count]
  p.x = x
  p.y = y
  p.vx = vx
  p.vy = vy
  p.ax = ax
  p.ay = ay
  p.s = s
  p.ds = ds
  p.life = life
  p.fulllife = life
  p.damping.x = damping_x
  p.damping.y = damping_y
  p.r = r or 255
  p.g = g or 255
  p.b = b or 255
  p.a = (a or 255) / 255
end


function Particles:update(dt)
  self.batch:clear()
  local i = 1
  while i < self.count+1 do
    local p = self.particles[i]

    p.life = p.life - dt
    if p.life <= 0 then
      local tmp = self.particles[self.count]
      self.particles[self.count] = p
      self.particles[i] = tmp
      self.count = self.count - 1
    else
      p.vx = p.vx + p.ax * dt
      p.vy = p.vy + p.ay * dt

      local dx = p.vx * p.damping.x * dt
      local dy = p.vy * p.damping.y * dt

      if coup.smath.sign(p.vx) == coup.smath.sign(p.vx - dx) then
        p.vx = p.vx - dx
      else 
        p.vx = 0
      end

      if coup.smath.sign(p.vy) == coup.smath.sign(p.vy - dy) then
        p.vy = p.vy - dy
      else 
        p.vy = 0
      end

      
      p.x = p.x + p.vx * dt
      p.y = p.y + p.vy * dt
      p.s = p.s + p.ds * dt
      i = i + 1
      self.batch:setColor(p.r,p.g,p.b,p.a*255 * (p.life / p.fulllife))
      self.batch:add(p.x, p.y, 0, p.s / 64, p.s / 64, 32, 32)
    end
  end
end


function Particles:render()
  love.graphics.draw(self.batch)
end


return Particles
