local TilePosUpdater = {}
TilePosUpdater.__index = TilePosUpdater


function TilePosUpdater:update(dt)
  self.world:foreachEntityWithComponents(function(e)
    local v = e:getComponent("TilePosition")
    local t = e:getComponent("Transform")
    local m = e:getComponent("TileMovement")

    local x1 = v.pos.x * self.tw
    local y1 = v.pos.y * self.th

    if m then
      if m.delay > 0 then
        m.delay = m.delay - dt
        if m.delay <= 0 then
          dt = -m.delay
        end
      end

      if m.delay <= 0 then
        m.tween = m.tween + m.speed * dt
        if m.tween >= 1 then
          m.tween = 1
          v.pos.x = m.pos.x
          v.pos.y = m.pos.y
          
          if m.after then
            m.after()
          end

          e:removeComponent("TileMovement")
          local elev = e:getComponent("Elevator")
          if elev then
            elev.engine:getComponent("Sprite").sprite:play("Engine_Idle")
          end
        end
        
        local x2 = m.pos.x * self.tw
        local y2 = m.pos.y * self.th

        x1 = coup.smath.interpolate(x1, x2, m.tween, m.ease)
        y1 = coup.smath.interpolate(y1, y2, m.tween, m.ease)
      end
    end

    local st = e:getComponent("PlayerState")
    if st then
      if st.dir == "r" or st.dir == "l"  or st.dir == "tl" or st.dir == "tr" then
        x1 = x1 - 16 
      elseif st.dir == "cr" then
        x1 = x1 - 64
      elseif st.dir == "dr" then
        x1 = x1 - 64
        y1 = y1 - 64
      elseif st.dir == "ddr" then
        x1 = x1 - 64
        y1 = y1 - 128 
      elseif st.dir == "dl" then
        y1 = y1 - 64
      elseif st.dir == "ddl" then
        y1 = y1 - 128
      elseif st.dir == "jr" then
        x1 = x1 - 128
      elseif st.dir == "pr" then
        x1 = x1 - 64
      end
    end



    if not t then
      e:makeComponent("Transform", x1, y2, 0)
    else
      t.pos.x = x1
      t.pos.y = y1
    end

  end, {"TilePosition"})
end


local function newTilePosUpdater(world, tw, th)
  local self = {
    world = world,
    tw = tw,
    th = th
  }

  return setmetatable(self, TilePosUpdater)
end


return newTilePosUpdater
